@@ -1,20 +1,35 @@
 from __future__ import annotations
 
 import inspect
-from typing import Any, Dict, Iterable, List, Mapping, Optional, Union, cast
+from typing import (
+    TYPE_CHECKING,
+    Any,
+    Dict,
+    Iterable,
+    List,
+    Mapping,
+    Optional,
+    Union,
+    cast,
+)
 
 from typing_extensions import Annotated, get_args, get_origin
 
 from strawberry.annotation import StrawberryAnnotation
+from strawberry.custom_scalar import ScalarDefinition, ScalarWrapper
 from strawberry.enum import EnumDefinition
+from strawberry.lazy_type import LazyType
 from strawberry.type import StrawberryList, StrawberryOptional, StrawberryType
-from strawberry.utils.mixins import GraphQLNameMixin
 
 from .exceptions import MultipleStrawberryArgumentsError, UnsupportedTypeError
 from .scalars import is_scalar
 from .types.types import TypeDefinition
 
 
+if TYPE_CHECKING:
+    from strawberry.schema.config import StrawberryConfig
+
+
 class _Unset:
     def __str__(self):
         return ""
@@ -39,7 +54,7 @@ class StrawberryArgumentAnnotation:
         self.name = name
 
 
-class StrawberryArgument(GraphQLNameMixin):
+class StrawberryArgument:
     def __init__(
         self,
         python_name: str,
@@ -94,7 +109,10 @@ class StrawberryArgument(GraphQLNameMixin):
 
 
 def convert_argument(
-    value: object, type_: Union[StrawberryType, type], auto_camel_case: bool = True
+    value: object,
+    type_: Union[StrawberryType, type],
+    scalar_registry: Dict[object, Union[ScalarWrapper, ScalarDefinition]],
+    config: StrawberryConfig,
 ) -> object:
     if value is None:
         return None
@@ -103,13 +121,16 @@ def convert_argument(
         return value
 
     if isinstance(type_, StrawberryOptional):
-        return convert_argument(value, type_.of_type)
+        return convert_argument(value, type_.of_type, scalar_registry, config)
 
     if isinstance(type_, StrawberryList):
         value_list = cast(Iterable, value)
-        return [convert_argument(x, type_.of_type) for x in value_list]
+        return [
+            convert_argument(x, type_.of_type, scalar_registry, config)
+            for x in value_list
+        ]
 
-    if is_scalar(type_):
+    if is_scalar(type_, scalar_registry):
         return value
 
     # Convert Enum fields to instances using the value. This is safe
@@ -117,6 +138,9 @@ def convert_argument(
     if isinstance(type_, EnumDefinition):
         return type_.wrapped_cls(value)
 
+    if isinstance(type_, LazyType):
+        return convert_argument(value, type_.resolve_type(), scalar_registry, config)
+
     if hasattr(type_, "_type_definition"):  # TODO: Replace with StrawberryInputObject
         type_definition: TypeDefinition = type_._type_definition  # type: ignore
 
@@ -126,11 +150,11 @@ def convert_argument(
 
         for field in type_definition.fields:
             value = cast(Mapping, value)
-            graphql_name = field.get_graphql_name(auto_camel_case)
+            graphql_name = config.name_converter.from_field(field)
 
             if graphql_name in value:
                 kwargs[field.python_name] = convert_argument(
-                    value[graphql_name], field.type, auto_camel_case
+                    value[graphql_name], field.type, scalar_registry, config
                 )
 
         type_ = cast(type, type_)
@@ -142,7 +166,8 @@ def convert_argument(
 def convert_arguments(
     value: Dict[str, Any],
     arguments: List[StrawberryArgument],
-    auto_camel_case: bool = True,
+    scalar_registry: Dict[object, Union[ScalarWrapper, ScalarDefinition]],
+    config: StrawberryConfig,
 ) -> Dict[str, Any]:
     """Converts a nested dictionary to a dictionary of actual types.
 
@@ -157,7 +182,7 @@ def convert_arguments(
     for argument in arguments:
         assert argument.python_name
 
-        name = argument.get_graphql_name(auto_camel_case)
+        name = config.name_converter.from_argument(argument)
 
         if name in value:
             current_value = value[name]
@@ -165,7 +190,8 @@ def convert_arguments(
             kwargs[argument.python_name] = convert_argument(
                 value=current_value,
                 type_=argument.type,
-                auto_camel_case=auto_camel_case,
+                config=config,
+                scalar_registry=scalar_registry,
             )
 
     return kwargs
