@@ -3,30 +3,26 @@ from __future__ import annotations
 import dataclasses
 from typing import (
     TYPE_CHECKING,
-    Iterable,
+    Any,
+    Callable,
     List,
     Mapping,
     Optional,
+    Sequence,
     Type,
     TypeVar,
     Union,
 )
 
-from strawberry.enum import EnumDefinition
-from strawberry.lazy_type import LazyType
 from strawberry.type import StrawberryType, StrawberryTypeVar
-from strawberry.utils.str_converters import capitalize_first
 from strawberry.utils.typing import is_generic as is_type_generic
 
 
 if TYPE_CHECKING:
+    from graphql import GraphQLResolveInfo
+
     from strawberry.field import StrawberryField
-
-
-@dataclasses.dataclass
-class FederationTypeParams:
-    keys: List[str] = dataclasses.field(default_factory=list)
-    extend: bool = False
+    from strawberry.schema_directive import StrawberrySchemaDirective
 
 
 @dataclasses.dataclass(eq=False)
@@ -36,8 +32,10 @@ class TypeDefinition(StrawberryType):
     is_interface: bool
     origin: Type
     description: Optional[str]
-    federation: FederationTypeParams
     interfaces: List["TypeDefinition"]
+    extend: bool
+    directives: Optional[Sequence[StrawberrySchemaDirective]]
+    is_type_of: Optional[Callable[[Any, GraphQLResolveInfo], bool]]
 
     _fields: List["StrawberryField"]
 
@@ -68,8 +66,6 @@ class TypeDefinition(StrawberryType):
     def copy_with(
         self, type_var_map: Mapping[TypeVar, Union[StrawberryType, type]]
     ) -> type:
-        name = self.get_name_from_types(type_var_map.values())
-
         fields = []
         for field in self.fields:
             # TODO: Logic unnecessary with StrawberryObject
@@ -86,13 +82,15 @@ class TypeDefinition(StrawberryType):
             fields.append(field)
 
         new_type_definition = TypeDefinition(
-            name=name,
+            name=self.name,
             is_input=self.is_input,
             origin=self.origin,
             is_interface=self.is_interface,
-            federation=self.federation,
+            directives=self.directives,
             interfaces=self.interfaces,
             description=self.description,
+            extend=self.extend,
+            is_type_of=self.is_type_of,
             _fields=fields,
             concrete_of=self,
             type_var_map=type_var_map,
@@ -113,29 +111,6 @@ class TypeDefinition(StrawberryType):
             (field for field in self.fields if field.python_name == python_name), None
         )
 
-    def get_name_from_types(self, types: Iterable[Union[StrawberryType, type]]) -> str:
-        from strawberry.union import StrawberryUnion
-
-        names: List[str] = []
-
-        for type_ in types:
-            if isinstance(type_, LazyType):
-                name = type_.type_name
-            elif isinstance(type_, EnumDefinition):
-                name = type_.name
-            elif isinstance(type_, StrawberryUnion):
-                name = type_.name
-            elif hasattr(type_, "_type_definition"):
-                field_type = type_._type_definition  # type: ignore
-
-                name = capitalize_first(field_type.name)
-            else:
-                name = capitalize_first(type_.__name__)  # type: ignore
-
-            names.append(name)
-
-        return "".join(names) + self.name
-
     @property
     def fields(self) -> List["StrawberryField"]:
         # TODO: rename _fields to fields and remove this property
@@ -183,15 +158,15 @@ class TypeDefinition(StrawberryType):
 
             # Check if the expected type matches the type found on the type_map
             real_concrete_type = type(getattr(root, generic_field.name))
+
+            # TODO: uniform type var map, at the moment we map object types
+            # to their class (not to TypeDefinition) while we map enum to
+            # the EnumDefinition class. This is why we do this check here:
+            if hasattr(real_concrete_type, "_enum_definition"):
+                real_concrete_type = real_concrete_type._enum_definition
+
             if real_concrete_type is not expected_concrete_type:
                 return False
 
         # All field mappings succeeded. This is a match
         return True
-
-
-@dataclasses.dataclass
-class FederationFieldParams:
-    provides: List[str] = dataclasses.field(default_factory=list)
-    requires: List[str] = dataclasses.field(default_factory=list)
-    external: bool = False
